@@ -8,9 +8,13 @@ Enhancements by James Stewart since 2021
 Library for communicating with the BES (BigFix) REST API.
 """
 
+import json
 import os
 import site
 import string
+import sys
+
+# import urllib3.poolmanager
 
 try:
     from urllib import parse
@@ -41,6 +45,71 @@ def sanitize_txt(*args):
     return tuple(sani_args)
 
 
+def elem2dict(node):
+    """
+    Convert an lxml.etree node tree into a dict.
+    https://gist.github.com/jacobian/795571?permalink_comment_id=2981870#gistcomment-2981870
+    """
+    result = {}
+
+    for element in node.iterchildren():
+        # Remove namespace prefix
+        key = element.tag.split("}")[1] if "}" in element.tag else element.tag
+
+        # Process element as tree element if the inner XML contains non-whitespace content
+        if element.text and element.text.strip():
+            value = element.text
+        else:
+            value = elem2dict(element)
+        if key in result:
+
+            if type(result[key]) is list:
+                result[key].append(value)
+            else:
+                tempvalue = result[key].copy()
+                result[key] = [tempvalue, value]
+        else:
+            result[key] = value
+    return result
+
+
+# # https://docs.python-requests.org/en/latest/user/advanced/#transport-adapters
+# class HTTPAdapterBiggerBlocksize(requests.adapters.HTTPAdapter):
+#     """custom HTTPAdapter for requests to override blocksize
+#     for Uploading or Downloading large files"""
+
+#     # override inti_poolmanager from regular HTTPAdapter
+#     # https://stackoverflow.com/questions/22915295/python-requests-post-and-big-content/22915488#comment125583017_22915488
+#     def init_poolmanager(self, connections, maxsize, block=False, **pool_kwargs):
+#         """Initializes a urllib3 PoolManager.
+
+#         This method should not be called from user code, and is only
+#         exposed for use when subclassing the
+#         :class:`HTTPAdapter <requests.adapters.HTTPAdapter>`.
+
+#         :param connections: The number of urllib3 connection pools to cache.
+#         :param maxsize: The maximum number of connections to save in the pool.
+#         :param block: Block when no free connections are available.
+#         :param pool_kwargs: Extra keyword arguments used to initialize the Pool Manager.
+#         """
+#         # save these values for pickling
+#         self._pool_connections = connections
+#         self._pool_maxsize = maxsize
+#         self._pool_block = block
+
+#         # This doesn't work until urllib3 is updated to a future version:
+#         # updating blocksize to be larger:
+#         # pool_kwargs["blocksize"] = 8 * 1024 * 1024
+
+#         self.poolmanager = urllib3.poolmanager.PoolManager(
+#             num_pools=connections,
+#             maxsize=maxsize,
+#             block=block,
+#             strict=True,
+#             **pool_kwargs,
+#         )
+
+
 class BESConnection:
     """BigFix RESTAPI connection abstraction class"""
 
@@ -166,6 +235,10 @@ class BESConnection:
         if not self.connected():
             self.get("login").request.raise_for_status()
 
+        # This doesn't work until urllib3 is updated to a future version:
+        # if self.connected():
+        #     self.session.mount(self.url("upload"), HTTPAdapterBiggerBlocksize())
+
         return self.connected()
 
     def logout(self):
@@ -203,6 +276,8 @@ class BESConnection:
             print("export_site_contents()")
         # Iterate Over All Site Content
         content = self.get("site/" + site_path + "/content")
+        if verbose:
+            print(content)
         if content.request.status_code == 200:
             print(
                 "Archiving %d items from %s..." % (content().countchildren(), site_path)
@@ -251,6 +326,22 @@ class BESConnection:
                     ) as bes_file:
                         bes_file.write(content.text.encode("utf-8"))
 
+    def export_all_sites(
+        self, include_external=False, export_folder="./", name_trim=70, verbose=False
+    ):
+        """export all bigfix sites to a folder"""
+        results_sites = self.get("sites")
+        if verbose:
+            print(results_sites)
+        if results_sites.request.status_code == 200:
+            for item in results_sites().iterchildren():
+                site_path = item.attrib["Resource"].split("/api/site/", 1)[1]
+                if include_external or "external/" not in site_path:
+                    print("Exporting Site:", site_path)
+                    self.export_site_contents(
+                        site_path, export_folder, name_trim, verbose
+                    )
+
     __call__ = login
     # https://stackoverflow.com/q/40536821/861745
     __enter__ = login
@@ -265,6 +356,8 @@ class RESTResult:
         self.text = request.text
         self._besxml = None
         self._besobj = None
+        self._besdict = None
+        self._besjson = None
 
         if (
             "content-type" in request.headers
@@ -311,6 +404,25 @@ class RESTResult:
 
         return self._besobj
 
+    @property
+    def besdict(self):
+        """property for python dict representation"""
+        if self._besdict is None:
+            if self.valid:
+                self._besdict = elem2dict(etree.fromstring(self.besxml))
+            else:
+                self._besdict = {"text": str(self)}
+
+        return self._besdict
+
+    @property
+    def besjson(self):
+        """property for json representation"""
+        if self._besjson is None:
+            self._besjson = json.dumps(self.besdict, indent=2)
+
+        return self._besjson
+
     def validate_xsd(self, doc):
         """validate results using XML XSDs"""
         try:
